use v6;
#  -- DO NOT EDIT --
# generated by: t/build.t 

class t::CSS::Aural::Spec::Actions {
    method expr-cue-after($/) { make $.list($/) }
    method expr-cue-before($/) { make $.list($/) }
    method generic-voice($/) { make $.rule($/) }
    method specific-voice($/) { make $.rule($/) }
}
