use v6;
#  -- DO NOT EDIT --
# generated by: t/build.t 
role t::CSS::Aural::Spec::Interface {
    method angle($/) { ... }
    method color($/) { ... }
    method frequency($/) { ... }
    method generic-voice($/) { ... }
    method identifier($/) { ... }
    method number($/) { ... }
    method percentage($/) { ... }
    method specific-voice($/) { ... }
    method string($/) { ... }
    method time($/) { ... }
    method uri($/) { ... }
}
